# Copyright (c) 2017 Andrea Frittoli
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

from .json.blacklists_client import BlacklistsClient
from .json.designate_limit_client import DesignateLimitClient
from .json.pool_client import PoolClient
from .json.ptr_client import PtrClient
from .json.quotas_client import QuotasClient
from .json.recordset_client import RecordsetClient
from .json.service_client import ServiceClient
from .json.shared_zones_client import SharedZonesClient
from .json.tld_client import TldClient
from .json.transfer_accepts_client import TransferAcceptClient
from .json.transfer_request_client import TransferRequestClient
from .json.tsigkey_client import TsigkeyClient
from .json.zones_client import ZonesClient
from .json.zone_exports_client import ZoneExportsClient
from .json.zone_imports_client import ZoneImportsClient
from .json.api_version_client import ApiVersionClient

__all__ = ['BlacklistsClient', 'DesignateLimitClient', 'PoolClient',
           'PtrClient', 'QuotasClient', 'RecordsetClient', 'ServiceClient',
           'SharedZonesClient', 'TldClient', 'TransferAcceptClient',
           'TransferRequestClient', 'TsigkeyClient', 'ZonesClient',
           'ZoneExportsClient', 'ZoneImportsClient', 'ApiVersionClient']



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from designate_tempest_plugin.tests import base
from designate_tempest_plugin import clients

LOG = logging.getLogger(__name__)


class TestDnsUnauthed(base.BaseDnsV2Test):

    client_manager = clients.ManagerV2Unauthed
    credentials = ["primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestDnsUnauthed, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestDnsUnauthed, cls).setup_clients()
        cls.zones_client = cls.os_primary.zones_client
        cls.recordset_client = cls.os_primary.recordset_client
        cls.tld_client = cls.os_primary.tld_client
        cls.pool_client = cls.os_primary.pool_client
        cls.blacklists_client = cls.os_primary.blacklists_client

    def _test_unauthed(self, client, method, args=None):
        client = getattr(self, client)
        method = getattr(client, method)
        args = args or []
        self.assertRaises(lib_exc.Unauthorized, method, *args)

    @decorators.idempotent_id('b18827ac-de92-11ed-8334-201e8823901f')
    def test_list_zones(self):
        self._test_unauthed('zones_client', 'list_zones')

    @decorators.idempotent_id('f60c32ce-de92-11ed-8334-201e8823901f')
    def test_show_zone(self):
        self._test_unauthed(
            'zones_client', 'show_zone',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('56e899c0-de93-11ed-8334-201e8823901f')
    def test_create_zone(self):
        self._test_unauthed(
            'zones_client', 'create_zone',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('5765af6e-de93-11ed-8334-201e8823901f')
    def test_update_zone(self):
        self._test_unauthed(
            'zones_client', 'update_zone',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('57b5cef4-de93-11ed-8334-201e8823901f')
    def test_delete_zone(self):
        self._test_unauthed(
            'zones_client', 'delete_zone',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('05099b62-de94-11ed-8334-201e8823901f')
    def test_list_recordsets(self):
        self._test_unauthed(
            'recordset_client', 'list_recordset',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('0573ca32-de94-11ed-8334-201e8823901f')
    def test_show_recordset(self):
        self._test_unauthed(
            'recordset_client', 'show_recordset',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90",
             "6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('05c0236e-de94-11ed-8334-201e8823901f')
    def test_create_recordset(self):
        self._test_unauthed(
            'recordset_client', 'create_recordset',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90",
             "6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('0600f628-de94-11ed-8334-201e8823901f')
    def test_update_recordset(self):
        self._test_unauthed(
            'recordset_client', 'update_recordset',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90",
             "6ef3b7f2-df39-43ef-9f37-ce2bc424ab90", {}])

    @decorators.idempotent_id('063c95b6-de94-11ed-8334-201e8823901f')
    def test_delete_recordset(self):
        self._test_unauthed(
            'recordset_client', 'delete_recordset',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90",
             "6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('ee9dae6c-de94-11ed-8334-201e8823901f')
    def test_list_tlds(self):
        self._test_unauthed('tld_client', 'list_tlds')

    @decorators.idempotent_id('eef1e5f4-de94-11ed-8334-201e8823901f')
    def test_show_tld(self):
        self._test_unauthed(
            'tld_client', 'show_tld',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('ef3ae024-de94-11ed-8334-201e8823901f')
    def test_create_tld(self):
        self._test_unauthed(
            'tld_client', 'create_tld',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('ef7cfda6-de94-11ed-8334-201e8823901f')
    def test_update_tld(self):
        self._test_unauthed(
            'tld_client', 'update_tld',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('efb982e4-de94-11ed-8334-201e8823901f')
    def test_delete_tld(self):
        self._test_unauthed(
            'tld_client', 'delete_tld',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('654e7596-de95-11ed-8334-201e8823901f')
    def test_list_blacklists(self):
        self._test_unauthed('blacklists_client', 'list_blacklists')

    @decorators.idempotent_id('658ea9cc-de95-11ed-8334-201e8823901f')
    def test_show_blacklist(self):
        self._test_unauthed(
            'blacklists_client', 'show_blacklist',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('65cbc2ee-de95-11ed-8334-201e8823901f')
    def test_create_blacklist(self):
        self._test_unauthed(
            'blacklists_client', 'create_blacklist',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('66032676-de95-11ed-8334-201e8823901f')
    def test_update_blacklist(self):
        self._test_unauthed(
            'blacklists_client', 'update_blacklist',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('66321184-de95-11ed-8334-201e8823901f')
    def test_delete_blacklist(self):
        self._test_unauthed(
            'blacklists_client', 'delete_blacklist',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('c7048d66-de95-11ed-8334-201e8823901f')
    def test_list_pools(self):
        self._test_unauthed('pool_client', 'list_pools')

    @decorators.idempotent_id('c74581cc-de95-11ed-8334-201e8823901f')
    def test_show_pool(self):
        self._test_unauthed(
            'pool_client', 'show_pool',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('c77d62f4-de95-11ed-8334-201e8823901f')
    def test_create_pool(self):
        self._test_unauthed(
            'pool_client', 'create_pool',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('c7ada040-de95-11ed-8334-201e8823901f')
    def test_update_pool(self):
        self._test_unauthed(
            'pool_client', 'update_pool',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])

    @decorators.idempotent_id('c7e07682-de95-11ed-8334-201e8823901f')
    def test_delete_pool(self):
        self._test_unauthed(
            'pool_client', 'delete_pool',
            ["6ef3b7f2-df39-43ef-9f37-ce2bc424ab90"])



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class DesignateLimit(base.BaseDnsV2Test):

    credentials = ["primary", "admin"]

    @classmethod
    def setup_clients(cls):
        super(DesignateLimit, cls).setup_clients()
        cls.limit_client = cls.os_primary.dns_v2.DesignateLimitClient()
        cls.zone_client = cls.os_primary.dns_v2.ZonesClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()
        if CONF.enforce_scope.designate:
            cls.admin_tld_client = cls.os_system_admin.dns_v2.TldClient()
        else:
            cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(DesignateLimit, cls).resource_setup()
        cls.project_limits = cls.limit_client.list_designate_limits()
        cls.tld = cls.admin_tld_client.create_tld(
            tld_name=dns_data_utils.rand_string(5))[1]

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.tld['id'])
        super(DesignateLimit, cls).resource_cleanup()

    @decorators.idempotent_id('3d1b09a2-b8be-11ec-86fe-201e8823901f')
    def test_max_zone_name_length(self):
        allowed_limit = self.project_limits[
            'max_zone_name_length'] - 1  # The final root null byte
        LOG.info(
            'Attempting to create a Zone of length:{}, expected: zone is'
            ' successfully created'.format(allowed_limit))
        zone_name = dns_data_utils.rand_dns_name_by_size(allowed_limit)
        # Use class TLD at the end of generated Zone Name
        zone_name = zone_name[:-(
                len(self.tld['name']) + 2)] + '.' + self.tld['name'] + '.'
        zone = self.zone_client.create_zone(
            name=zone_name, wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zone_client, zone['id'])

        LOG.info(
            'Attempting to create a Zone of length:{}, expected: zone is '
            'failed to be created'.format(allowed_limit + 1))
        zone_name = dns_data_utils.rand_dns_name_by_size(allowed_limit + 1)
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.zone_client.create_zone,
            name=zone_name
        )

    @decorators.idempotent_id('86646744-b98a-11ec-b3a4-201e8823901f')
    def test_max_recordset_name_length(self):
        # The full recordset name is a combination of its short name
        # and the zone name. e.g., "www" + "example.com." = "www.example.com."
        # In this section, we'll set the lengths needed for testing.
        allowed_recordset_limit = self.project_limits[
            'max_recordset_name_length'] - 1  # The final root null byte
        reserved_recordset_length = 10  # Reserved for recordset's host part.
        zone_name_size = allowed_recordset_limit - reserved_recordset_length
        zone_name = dns_data_utils.rand_dns_name_by_size(
            name_size=zone_name_size)
        # Use class TLD at the end of generated Zone Name
        zone_name = zone_name[:-(
                len(self.tld['name']) + 2)] + '.' + self.tld['name'] + '.'
        max_valid_record_name_size = allowed_recordset_limit - len(zone_name)

        LOG.info('Create a Zone')
        zone = self.zone_client.create_zone(
            name=zone_name, wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zone_client, zone['id'])

        LOG.info('Recordset name of length:{} is successfully '
                 'created'.format(allowed_recordset_limit))
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A',
            zone_name=zone_name,
            # Reserve 1 char for the dot between the record name and zone name.
            name=dns_data_utils.rand_string(
                max_valid_record_name_size - 1) + '.' + zone_name)
        recordset = self.recordset_client.create_recordset(
            zone['id'], recordset_data)[1]
        self.addCleanup(
            self.wait_recordset_delete, self.recordset_client,
            zone['id'], recordset['id'])
        LOG.info(
            'Attempting to create a Recordset of length:{}, expected:'
            ' Recordset is failed to be created'.format(
                allowed_recordset_limit + 1))
        recordset_data = dns_data_utils.rand_recordset_data(
            record_type='A',
            zone_name=zone_name,
            name=dns_data_utils.rand_string(
                max_valid_record_name_size) + '.' + zone_name)

        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.recordset_client.create_recordset,
            zone_uuid=zone['id'],
            recordset_data=recordset_data)



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from tempest import test
from tempest import config
from tempest.lib.common.utils import test_utils as utils

from designate_tempest_plugin.services.dns.query.query_client import (
    QueryClient)
from designate_tempest_plugin.tests import rbac_utils


CONF = config.CONF


class AssertRaisesDns(test.BaseTestCase):
    def __init__(self, test_class, exc, type_, code):
        self.test_class = test_class
        self.exc = exc
        self.type_ = type_
        self.code = code

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is None:
            try:
                exc_name = self.exc.__name__
            except AttributeError:
                exc_name = str(self.exc)
            raise self.failureException(
                "{0} not raised".format(exc_name))

        if issubclass(exc_type, self.exc):
            self.test_class.assertEqual(
                self.code, exc_value.resp_body['code'])

            self.test_class.assertEqual(
                    self.type_, exc_value.resp_body['type'])

            return True

        # Unexpected exceptions will be reraised
        return False


class BaseDnsTest(rbac_utils.RBACTestsMixin, test.BaseTestCase):
    """Base class for DNS tests."""

    # NOTE(andreaf) credentials holds a list of the credentials to be allocated
    # at class setup time. Credential types can be 'primary', 'alt', 'admin' or
    # a list of roles - the first element of the list being a label, and the
    # rest the actual roles.
    # NOTE(kiall) primary will result in a manager @ cls.os_primary, alt will
    # have cls.os_alt, and admin will have cls.os_admin.
    # NOTE(johnsom) We will allocate most credentials here so that each test
    # can test for allowed and disallowed RBAC policies.
    credentials = ['admin', 'primary', 'alt']
    if CONF.dns_feature_enabled.enforce_new_defaults:
        credentials.extend(['project_member', 'project_reader'])

    # A tuple of credentials that will be allocated by tempest using the
    # 'credentials' list above. These are used to build RBAC test lists.
    allocated_creds = []
    for cred in credentials:
        if isinstance(cred, list):
            allocated_creds.append('os_roles_' + cred[0])
        else:
            allocated_creds.append('os_' + cred)
    # Tests shall not mess with the list of allocated credentials
    allocated_credentials = tuple(allocated_creds)

    @classmethod
    def skip_checks(cls):
        super(BaseDnsTest, cls).skip_checks()

        if not CONF.service_available.designate:
            skip_msg = ("%s skipped as designate is not available"
                        % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(BaseDnsTest, cls).setup_clients()
        # The Query Client is not an OpenStack client which means
        # we should not set it up through the tempest client manager.
        # Set it up here so all tests have access to it.
        cls.query_client = QueryClient(
            nameservers=CONF.dns.nameservers,
            query_timeout=CONF.dns.query_timeout,
            build_interval=CONF.dns.build_interval,
            build_timeout=CONF.dns.build_timeout,
        )
        # Most tests need a "primary" zones client and we need it for the
        # API version check, so create one instance here.
        cls.zones_client = cls.os_primary.dns_v2.ZonesClient()

    @classmethod
    def resource_setup(cls):
        """Setup resources needed by the tests."""
        super(BaseDnsTest, cls).resource_setup()

        # The credential does not matter here.
        cls.api_version = cls.zones_client.get_max_api_version()

    def assertExpected(self, expected, actual, excluded_keys):
        for key, value in expected.items():
            if key not in excluded_keys:
                self.assertIn(key, actual)
                self.assertEqual(value, actual[key], key)

    def assertRaisesDns(self, exc, type_, code, callable_=None, *args,
                        **kwargs):
        """
        Checks the response that a api call with a exception contains the
        expected data

        Usable as both a ordinary function, and a context manager
        """
        context = AssertRaisesDns(self, exc, type_, code)

        if callable_ is None:
            return context

        with context:
            callable_(*args, **kwargs)

    def transfer_request_delete(self, transfer_client, transfer_request_id):
        return utils.call_and_ignore_notfound_exc(
            transfer_client.delete_transfer_request, transfer_request_id)

    def wait_zone_delete(self, zone_client, zone_id, **kwargs):
        self._delete_zone(zone_client, zone_id, **kwargs)
        utils.call_until_true(self._check_zone_deleted,
                              CONF.dns.build_timeout,
                              CONF.dns.build_interval,
                              zone_client,
                              zone_id)

    def wait_recordset_delete(self, recordset_client, zone_id,
                              recordset_id, **kwargs):
        self._delete_recordset(
            recordset_client, zone_id, recordset_id, **kwargs)
        utils.call_until_true(self._check_recordset_deleted,
                              CONF.dns.build_timeout,
                              CONF.dns.build_interval,
                              recordset_client,
                              zone_id,
                              recordset_id)

    def unset_ptr(self, ptr_client, fip_id, **kwargs):
        return utils.call_and_ignore_notfound_exc(
            ptr_client.unset_ptr_record, fip_id, **kwargs)

    def _delete_zone(self, zone_client, zone_id, **kwargs):
        return utils.call_and_ignore_notfound_exc(zone_client.delete_zone,
                                                  zone_id, **kwargs)

    def _check_zone_deleted(self, zone_client, zone_id):
        return utils.call_and_ignore_notfound_exc(zone_client.show_zone,
                                                  zone_id) is None

    def _delete_recordset(self, recordset_client, zone_id,
                          recordset_id, **kwargs):
        return utils.call_and_ignore_notfound_exc(
            recordset_client.delete_recordset,
            zone_id, recordset_id, **kwargs)

    def _check_recordset_deleted(
            self, recordset_client, zone_id, recordset_id):
        return utils.call_and_ignore_notfound_exc(
            recordset_client.show_recordset, zone_id, recordset_id) is None


class BaseDnsV2Test(BaseDnsTest):
    """Base class for DNS V2 API tests."""

    all_projects_header = {'X-Auth-All-Projects': True}
    managed_records = {'x-designate-edit-managed-records': True}

    @classmethod
    def skip_checks(cls):
        super(BaseDnsV2Test, cls).skip_checks()

        if not CONF.dns_feature_enabled.api_v2:
            skip_msg = ("%s skipped as designate v2 API is not available"
                        % cls.__name__)
            raise cls.skipException(skip_msg)


class BaseDnsAdminTest(BaseDnsTest):
    """Base class for DNS Admin API tests."""

    @classmethod
    def skip_checks(cls):
        super(BaseDnsAdminTest, cls).skip_checks()
        if not CONF.dns_feature_enabled.api_admin:
            skip_msg = ("%s skipped as designate admin API is not available"
                        % cls.__name__)
            raise cls.skipException(skip_msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.services.dns.query.query_client import (
    SingleQueryClient)

LOG = logging.getLogger(__name__)

CONF = config.CONF


class RecordsetsTest(base.BaseDnsV2Test):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_clients(cls):
        super(RecordsetsTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.RecordsetClient()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()

    @classmethod
    def resource_setup(cls):
        super(RecordsetsTest, cls).resource_setup()

        zone_id = CONF.dns.zone_id
        if zone_id:
            LOG.info('Retrieve info from a zone')
            zone = cls.zones_client.show_zone(zone_id)[1]
        else:
            # Make sure we have an allowed TLD available
            tld_name = dns_data_utils.rand_zone_name(name="RecordsetsTest")
            cls.tld_name = f".{tld_name}"
            cls.class_tld = cls.admin_tld_client.create_tld(
                tld_name=tld_name[:-1])
            LOG.info('Create a new zone')
            zone_name = dns_data_utils.rand_zone_name(
                name="recordsets_test_setup", suffix=cls.tld_name)
            zone = cls.zones_client.create_zone(name=zone_name)[1]
            cls.addClassResourceCleanup(
                test_utils.call_and_ignore_notfound_exc,
                cls.zones_client.delete_zone, zone['id'])

        LOG.info('Ensure we respond with ACTIVE')
        waiters.wait_for_zone_status(cls.zones_client, zone['id'], 'ACTIVE')

        cls.zone = zone

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(RecordsetsTest, cls).resource_cleanup()

    def _test_create_and_delete_records_on_existing_zone(
            self, name, type, records):
        if name is not None:
            recordset_name = name + "." + self.zone['name']

        else:
            recordset_name = self.zone['name']

        recordset_data = {
            'name': recordset_name,
            'type': type,
            'records': records,
        }

        LOG.info('Create a Recordset on the existing zone')
        recordset = self.recordset_client.create_recordset(
            self.zone['id'], recordset_data)[1]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.recordset_client.delete_recordset,
                        self.zone['id'], recordset['id'])

        LOG.info('Ensure we respond with PENDING')
        self.assertEqual('PENDING', recordset['status'])

        LOG.info('Wait until the recordset is active and propagated')
        waiters.wait_for_recordset_status(self.recordset_client,
                                          self.zone['id'], recordset['id'],
                                          'ACTIVE')
        waiters.wait_for_query(
            self.query_client, recordset_data['name'], type)

        LOG.info('Delete the recordset')
        body = self.recordset_client.delete_recordset(
            self.zone['id'], recordset['id'])[1]

        LOG.info('Ensure we respond with DELETE+PENDING')
        self.assertEqual('DELETE', body['action'])
        self.assertEqual('PENDING', body['status'])

        LOG.info('Ensure successful deletion of Recordset from:'
                 ' Designate and Backends')
        self.assertRaises(lib_exc.NotFound,
                          lambda: self.recordset_client.show_recordset(
                              self.zone['id'], recordset['id']))
        waiters.wait_for_query(
            self.query_client, recordset_data['name'], type, found=False)

    @decorators.attr(type='slow')
    @decorators.idempotent_id('4664ed66-9ff1-45f2-9e60-d4913195c505')
    def test_create_and_delete_records_on_existing_zone_01_A(self):
        self._test_create_and_delete_records_on_existing_zone(
            "www", "A", ["192.0.2.1", "192.0.2.2", "192.0.2.3"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('cecd9f20-0b62-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_02_AAAA(self):
        self._test_create_and_delete_records_on_existing_zone(
            "www", "AAAA", ["2001:db8::1", "2001:db8::1", "2001:db8::"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('f5368d7a-0b62-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_03_SRV(self):
        self._test_create_and_delete_records_on_existing_zone(
            "_sip._tcp", "SRV", ["10 60 5060 server1.example.com.",
                    "20 60 5060 server2.example.com.",
                    "20 30 5060 server3.example.com."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('74ff9efc-0b63-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_04_SRV(self):
        self._test_create_and_delete_records_on_existing_zone(
            "_sip._udp", "SRV", ["10 60 5060 server1.example.com.",
                    "10 60 5060 server2.example.com.",
                    "20 30 5060 server3.example.com."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('82a14a2e-0b63-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_05_CNAME(self):
        self._test_create_and_delete_records_on_existing_zone(
            "alias-of-target", "CNAME", ["target.example.org."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('ae7a295e-0b63-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_06_MX(self):
        self._test_create_and_delete_records_on_existing_zone(
            None, "MX", ["10 mail1.example.org.",
                    "20 mail2.example.org."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('f9aa8512-0b64-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_07_MX(self):
        self._test_create_and_delete_records_on_existing_zone(
            "under", "MX", ["10 mail.example.org."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('fa6cbd12-0b64-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_08_SSHFP(self):
        self._test_create_and_delete_records_on_existing_zone(
            "www", "SSHFP", ["2 1 123456789abcdef67890123456789abcdef67890"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('fa124a1c-0b64-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_09_TXT(self):
        self._test_create_and_delete_records_on_existing_zone(
            "www", "TXT", ["\"Any Old Text Goes Here\""])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('3e347c28-0b66-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_10_SPF(self):
        self._test_create_and_delete_records_on_existing_zone(
            "*.sub", "SPF", ["\"v=spf1; a -all\""])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('88f6c2ac-0b66-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_11_PTR(self):
        self._test_create_and_delete_records_on_existing_zone(
            "PTR_Record_IPV4", "PTR", ["34.216.184.93.in-addr.arpa."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('b9591eea-0b66-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_12_PTR(self):
        self._test_create_and_delete_records_on_existing_zone(
            "PTR_Record_IPV6", "PTR",
            ["6.4.9.1.8.c.5.2.3.9.8.1.8.4.2.0.1.0.0.0.0.2.2.0.0.0.8"
             ".2.6.0.6.2.ip6.arpa."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('c98cd9b4-0b66-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_13_CAA(self):
        self._test_create_and_delete_records_on_existing_zone(
            "CAA_Record", "CAA", ["0 issue letsencrypt.org"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('f78d6e8c-0b66-11ee-bbcc-201e8823901f')
    def test_create_and_delete_records_on_existing_zone_14_NAPTR(self):
        self._test_create_and_delete_records_on_existing_zone(
            "NAPTR_Record", "NAPTR",
            ["0 0 S SIP+D2U !^.*$!sip:customer-service@example"
             ".com! _sip._udp.example.com."])

    @testtools.skipUnless(
        config.CONF.dns.nameservers,
        "Config option dns.nameservers is missing or empty")
    def _test_update_records_propagated_to_backends(self, name, type, records):
        if name:
            recordset_name = name + "." + self.zone['name']
        else:
            recordset_name = self.zone['name']

        orig_ttl = 666
        updated_ttl = 777
        recordset_data = {
            'name': recordset_name,
            'type': type,
            'records': records,
            'ttl': orig_ttl
        }

        LOG.info('Create a Recordset on the existing zone')
        recordset = self.recordset_client.create_recordset(
            self.zone['id'], recordset_data, wait_until=const.ACTIVE)[1]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.recordset_client.delete_recordset,
                        self.zone['id'], recordset['id'])

        LOG.info('Update a Recordset on the existing zone')
        recordset_data['ttl'] = updated_ttl
        self.recordset_client.update_recordset(
            self.zone['id'], recordset['id'],
            recordset_data, wait_until=const.ACTIVE)

        LOG.info('Per Nameserver "dig" for a record until either:'
                 ' updated TTL is detected or build timeout has reached')
        for ns in config.CONF.dns.nameservers:
            start = time.time()
            while True:
                ns_obj = SingleQueryClient(ns, config.CONF.dns.query_timeout)
                ns_record = ns_obj.query(
                    self.zone['name'], rdatatype=recordset_data['type'])
                if str(updated_ttl) in str(ns_record):
                    return
                if time.time() - start >= config.CONF.dns.build_timeout:
                    raise lib_exc.TimeoutException(
                        'Failed, updated TTL:{} for the record was not'
                        ' detected on Nameserver:{} within a timeout of:{}'
                        ' seconds.'.format(
                            updated_ttl, ns, config.CONF.dns.build_timeout))

    # These tests were unrolled from DDT to allow accurate tracking by
    # idempotent_id's. The naming convention for the tests has been preserved.
    @decorators.attr(type='slow')
    @decorators.idempotent_id('cbf756b0-ba64-11ec-93d4-201e8823901f')
    def test_update_records_propagated_to_backends_01_A(self):
        self._test_update_records_propagated_to_backends(
            "www", "A", ["192.0.2.1", "192.0.2.2", "192.0.2.3"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('258f7f57-9a74-4e72-bbfb-c709c411af14')
    def test_update_records_propagated_to_backends_02_AAAA(self):
        self._test_update_records_propagated_to_backends(
            "www", "AAAA", ["2001:db8::1", "2001:db8::1", "2001:db8::"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('304adbc5-668a-457e-9496-8efd20b8ae82')
    def test_update_records_propagated_to_backends_03_SRV_TCP(self):
        self._test_update_records_propagated_to_backends(
            "_sip._tcp", "SRV", ["10 60 5060 server1.example.com.",
                                 "20 60 5060 server2.example.com.",
                                 "20 30 5060 server3.example.com."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('bd1283b3-423c-4bb9-8c4f-a205f31f1c2d')
    def test_update_records_propagated_to_backends_04_SRV_UDP(self):
        self._test_update_records_propagated_to_backends(
            "_sip._udp", "SRV", ["10 60 5060 server1.example.com.",
                                 "10 60 5060 server2.example.com.",
                                 "20 30 5060 server3.example.com."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('8b53ae20-d096-4651-a6cf-efd7c98ae8d1')
    def test_update_records_propagated_to_backends_05_CNAME(self):
        self._test_update_records_propagated_to_backends(
            "alias-of-target", "CNAME", ["target.example.org."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('0fd0046a-ac5a-468d-94b3-8a6bde790589')
    def test_update_records_propagated_to_backends_06_MX_at_APEX(self):
        self._test_update_records_propagated_to_backends(
            None, "MX", ["10 mail1.example.org.",
                         "20 mail2.example.org."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('31176def-3f95-459d-8bdd-b9994335b2d9')
    def test_update_records_propagated_to_backends_07_MX_under_APEX(self):
        self._test_update_records_propagated_to_backends(
            "under", "MX", ["10 mail.example.org."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('0009d787-c590-4149-9f30-082195326fad')
    def test_update_records_propagated_to_backends_08_SSHFP(self):
        self._test_update_records_propagated_to_backends(
            "www", "SSHFP", ["2 1 123456789abcdef67890123456789abcdef67890"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('af7cec16-dfad-4071-aa05-cafa60bf12a5')
    def test_update_records_propagated_to_backends_09_TXT(self):
        self._test_update_records_propagated_to_backends(
            "www", "TXT", ["\"Any Old Text Goes Here\""])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('b3fd1f77-c318-4ab0-b18d-34611e51e9e4')
    def test_update_records_propagated_to_backends_10_SPF(self):
        self._test_update_records_propagated_to_backends(
            "*.sub", "SPF", ["\"v=spf1; a -all\""])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('c310b94b-f3a5-4d26-bab6-2529e6f29fbf')
    def test_update_records_propagated_to_backends_11_PTR_IPV4(self):
        self._test_update_records_propagated_to_backends(
            "PTR_Record_IPV4", "PTR", ["34.216.184.93.in-addr.arpa."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('3e31e406-621f-4f89-b401-b6f38aa63347')
    def test_update_records_propagated_to_backends_12_PTR_IPV6(self):
        self._test_update_records_propagated_to_backends(
            "PTR_Record_IPV6", "PTR",
            ["6.4.9.1.8.c.5.2.3.9.8.1.8.4.2.0.1.0.0.0.0.2.2.0.0.0.8.2.6.0.6.2"
             ".ip6.arpa."])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('6fd96280-fb62-4eaf-81f9-609cdb7c126e')
    def test_update_records_propagated_to_backends_13_CAA_Record(self):
        self._test_update_records_propagated_to_backends(
            "CAA_Record", "CAA", ["0 issue letsencrypt.org"])

    @decorators.attr(type='slow')
    @decorators.idempotent_id('45a11efe-bee3-4896-ab7c-daee1cb5eb3a')
    def test_update_records_propagated_to_backends_14_NAPTR_Record(self):
        self._test_update_records_propagated_to_backends(
            "NAPTR_Record", "NAPTR",
            ["0 0 S SIP+D2U !^.*$!sip:customer-service@example.com! "
             "_sip._udp.example.com."])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.services.dns.v2.json import base


class RecordsetClient(base.DnsClientV2Base):
    """API V2 Tempest REST client for Recordset API"""

    SHOW_STATUS_CODES = [200, 301]

    def __init__(self, auth_provider, service, region,
                 endpoint_type='publicURL',
                 build_interval=1, build_timeout=60,
                 disable_ssl_certificate_validation=False, ca_certs=None,
                 trace_requests='', name=None, http_timeout=None,
                 proxy_url=None):
        super(RecordsetClient, self).__init__(
                auth_provider, service, region, endpoint_type, build_interval,
                build_timeout, disable_ssl_certificate_validation, ca_certs,
                trace_requests, name, http_timeout, proxy_url,
                follow_redirects=False)

    @base.handle_errors
    def create_recordset(self, zone_uuid, recordset_data,
                         params=None, headers=None, wait_until=False):
        """Create a recordset for the specified zone.

        :param zone_uuid: Unique identifier of the zone in UUID format..
        :param recordset_data: A dictionary that represents the recordset
                               data.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the created zone.
        """
        if headers:
            resp, body = self._create_request(
                "/zones/{0}/recordsets".format(zone_uuid), params=params,
                data=recordset_data, extra_headers=True, headers=headers)
        else:
            resp, body = self._create_request(
                "/zones/{0}/recordsets".format(zone_uuid), params=params,
                data=recordset_data)

        # Create Recordset should Return a HTTP 202
        self.expected_success(202, resp.status)

        if wait_until:
            waiters.wait_for_recordset_status(
                self, zone_uuid, body['id'], wait_until, headers=headers)

        return resp, body

    @base.handle_errors
    def update_recordset(self, zone_uuid, recordset_uuid,
                         recordset_data, params=None,
                         headers=None, extra_headers=None, wait_until=False):
        """Update the recordset related to the specified zone.
        :param zone_uuid: Unique identifier of the zone in UUID format.
        :param recordset_uuid: Unique identifier of the recordset in UUID
                               format.
        :param recordset_data: A dictionary that represents the recordset
                               data.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :param extra_headers (bool): Boolean value than indicates if the
                                     headers returned by the get_headers()
                                     method are to be used but additional
                                     headers are needed in the request
                                     pass them in as a dict.
        :param wait_until: Block until the recordset reaches the
                           desired status
        :return: A tuple with the server response and the created zone.
        """
        resp, body = self._put_request(
            'zones/{0}/recordsets'.format(zone_uuid), recordset_uuid,
            data=recordset_data, params=params,
            headers=headers, extra_headers=extra_headers)

        # Update Recordset should Return a HTTP 202, or a 200 if the recordset
        # is already active
        self.expected_success([200, 202], resp.status)

        if wait_until:
            waiters.wait_for_recordset_status(
                self, zone_uuid, body['id'], wait_until)

        return resp, body

    @base.handle_errors
    def show_recordset(self, zone_uuid, recordset_uuid,
                       params=None, headers=None):
        """Gets a specific recordset related to a specific zone.
        :param zone_uuid: Unique identifier of the zone in UUID format.
        :param recordset_uuid: Unique identifier of the recordset in
                               UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized recordset as a list.
        """
        return self._show_request(
            'zones/{0}/recordsets'.format(zone_uuid), recordset_uuid,
            params=params, headers=headers)

    @base.handle_errors
    def delete_recordset(self, zone_uuid, recordset_uuid, params=None,
                         headers=None):
        """Deletes a recordset related to the specified zone UUID.
        :param zone_uuid: The unique identifier of the zone.
        :param recordset_uuid: The unique identifier of the record in
                               uuid format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request(
            'zones/{0}/recordsets'.format(zone_uuid), recordset_uuid,
            params=params, headers=headers)

        # Delete Recordset should Return a HTTP 202
        self.expected_success(202, resp.status)

        return resp, body

    @base.handle_errors
    def list_recordset(self, uuid, params=None, headers=None):
        """List recordsets related to the specified zone.
        :param uuid: Unique identifier of the zone in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized recordset as a list.
        """
        return self._list_request(
            'zones/{0}/recordsets'.format(uuid),
            params=params, headers=headers)

    @base.handle_errors
    def show_zones_recordset(self, recordset_uuid, params=None):
        """Gets a single recordset, using the cross_zone endpoint
        :param recordset_uuid: Unique identifier of the recordset in UUID
                               format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._show_request(
            'recordsets', recordset_uuid,
            params=params)

        # Show recordsets/id should return a HTTP 301
        self.expected_success(301, resp.status)

        return resp, body

    @base.handle_errors
    def list_zones_recordsets(self, params=None):
        """List recordsets across all zones.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: Serialized recordset as a list.
        """
        return self._list_request(
            'recordsets', params=params)

    @base.handle_errors
    def list_owned_recordsets(self, params=None, headers=None):
        """Lists recordsets for all projects in Designate.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :param headers: (dict): The headers to use for the request.
        :return: Serialized recordset as a list.
        """
        return self._list_request(
            'recordsets', params=params, headers=headers)[1]['recordsets']



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from oslo_utils import versionutils
from tempest import config
from tempest.lib import decorators
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc


from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseZonesTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'version', 'links',
                    'status', 'action']

    @classmethod
    def setup_clients(cls):
        super(BaseZonesTest, cls).setup_clients()
        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseZonesTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseZonesTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseZonesTest, cls).resource_cleanup()


class ZonesTest(BaseZonesTest):

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesTest, cls).setup_clients()
        cls.pool_client = cls.os_admin.dns_v2.PoolClient()
        cls.recordset_client = cls.os_primary.dns_v2.RecordsetClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()
        cls.share_zone_client = cls.os_primary.dns_v2.SharedZonesClient()

    @decorators.idempotent_id('9d2e20fc-e56f-4a62-9c61-9752a9ec615c')
    def test_create_zones(self):
        # Create a PRIMARY zone
        LOG.info('Create a PRIMARY zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_zones_primary", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual(const.CREATE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        # Get the Name Servers (hosts) created in PRIMARY zone
        nameservers = self.zones_client.show_zone_nameservers(zone['id'])[1]
        nameservers = [dic['hostname'] for dic in nameservers['nameservers']]

        # Create a SECONDARY zone
        LOG.info('Create a SECONDARY zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_zones_secondary", suffix=self.tld_name)
        zone = self.zones_client.create_zone(
            name=zone_name, zone_type=const.SECONDARY_ZONE_TYPE,
            primaries=nameservers)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual(const.CREATE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        # Test with no extra header overrides (sudo-project-id)
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')

        self.check_CUD_RBAC_enforcement('ZonesClient', 'create_zone',
                                        expected_allowed, False)

        # Test with x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'ZonesClient', 'create_zone', expected_allowed, False,
            project_id=self.zones_client.project_id)

    @decorators.idempotent_id('ec150c22-f52e-11eb-b09b-74e5f9e2a801')
    def test_create_zone_validate_recordsets_created(self):
        # Create a PRIMARY zone and wait till it's Active
        LOG.info('Create a PRIMARY zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="create_zone_validate_recordsets", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                       wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual(const.CREATE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        LOG.info('Ensure that SOA and NS recordsets types has been created.')
        recordsets = self.recordset_client.list_recordset(
            zone['id'])[1]['recordsets']
        types = [rec['type'] for rec in recordsets]
        expected_types = ['SOA', 'NS']
        for exp_type in expected_types:
            self.assertIn(
                exp_type, types,
                'Failed, expected recordset type:{} was'
                ' not created'.format(exp_type))

    @decorators.idempotent_id('02ca5d6a-86ce-4f02-9d94-9e5db55c3055')
    def test_show_zone(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_zones", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Fetch the zone')
        body = self.zones_client.show_zone(zone['id'])[1]

        LOG.info('Ensure the fetched response matches the created zone')
        self.assertExpected(zone, body, self.excluded_keys)

        # Test with no extra header overrides (all_projects, sudo-project-id)
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member',
                                     'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'ZonesClient', 'show_zone', expected_allowed, True, zone['id'])

        # Test with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ZonesClient', 'show_zone', expected_allowed, False, zone['id'],
            headers=self.all_projects_header)
        self.check_list_show_RBAC_enforcement(
            'ZonesClient', 'show_zone', expected_allowed, False, zone['id'],
            headers={'x-auth-sudo-project-id': self.zones_client.project_id})

    @decorators.idempotent_id('81bff0fb-a5d1-4c64-84db-56ca751c17fc')
    def test_show_shared_zone(self):
        if not versionutils.is_compatible('2.1', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'Zone share tests require Designate API version 2.1 or newer. '
                'Skipping test_show_shared_zone test.')

        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_shared_zone", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Share the zone with alt')
        shared_zone = self.share_zone_client.create_zone_share(
            zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        zone['id'], shared_zone['id'])

        LOG.info('Fetch the zone as alt')
        body = self.alt_zone_client.show_zone(zone['id'])[1]

        # Account for the zone now being shared
        zone['shared'] = True

        LOG.info('Ensure the fetched response matches the created zone')
        self.assertExpected(zone, body, self.excluded_keys)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('a4791906-6cd6-4d27-9f15-32273db8bb3d')
    def test_delete_zone(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="delete_zones", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement('ZonesClient', 'delete_zone',
                                        expected_allowed, True, zone['id'])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement('ZonesClient', 'delete_zone',
                                        expected_allowed, False, zone['id'],
                                        headers=self.all_projects_header)
        self.check_CUD_RBAC_enforcement(
            'ZonesClient', 'delete_zone', expected_allowed, False, zone['id'],
            headers={'x-auth-sudo-project-id': self.zones_client.project_id})

        LOG.info('Delete the zone')
        body = self.zones_client.delete_zone(zone['id'])[1]

        LOG.info('Ensure we respond with DELETE+PENDING')
        self.assertEqual(const.DELETE, body['action'])
        self.assertEqual(const.PENDING, body['status'])

    @decorators.idempotent_id('bf2ee5c1-67b5-47dc-9902-ddb5b0e03e37')
    def test_delete_zone_with_shares(self):

        if not versionutils.is_compatible('2.1', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'Zone share tests require Designate API version 2.1 or newer. '
                'Skipping test_delete_zone_with_shares test.')

        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="delete_zones_with_shares", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        shared_zone = self.share_zone_client.create_zone_share(
            zone['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        zone['id'], shared_zone['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info('Attempt to delete the zone with shares')
        self.assertRaises(lib_exc.BadRequest, self.zones_client.delete_zone,
                          zone['id'])

        LOG.info('Make sure the zone share is still present')
        check_share = self.share_zone_client.show_zone_share(
            zone['id'], shared_zone['id'])[1]
        self.assertEqual(shared_zone['id'], check_share['id'])

        LOG.info('Delete the zone using delete-shares')
        body = self.zones_client.delete_zone(
            zone['id'], delete_shares=True)[1]

        LOG.info('Ensure we respond with DELETE+PENDING')
        self.assertEqual(const.DELETE, body['action'])
        self.assertEqual(const.PENDING, body['status'])

        self.assertRaises(lib_exc.NotFound,
                          self.share_zone_client.show_zone_share,
                          zone['id'], shared_zone['id'])

    @decorators.idempotent_id('5bfa3cfe-5bc8-443b-bf48-cfba44cbb247')
    def test_list_zones(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_zones", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('List zones')
        body = self.zones_client.list_zones()[1]

        # TODO(kiall): We really want to assert that our newly created zone is
        #              present in the response.
        self.assertGreater(len(body['zones']), 0)

        # Test RBAC - Users that are allowed to call list, but should get
        #             zero zones.
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed = ['os_admin']
        else:
            expected_allowed = ['os_alt']

        self.check_list_RBAC_enforcement_count(
            'ZonesClient', 'list_zones', expected_allowed, 0)

        # Test that users who should see the zone, can see it.
        expected_allowed = ['os_primary']

        self.check_list_IDs_RBAC_enforcement(
            'ZonesClient', 'list_zones', expected_allowed, [zone['id']])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'ZonesClient', 'list_zones', expected_allowed, [zone['id']],
            headers=self.all_projects_header)
        self.check_list_IDs_RBAC_enforcement(
            'ZonesClient', 'list_zones', expected_allowed, [zone['id']],
            headers={'x-auth-sudo-project-id': self.zones_client.project_id})

    @decorators.idempotent_id('ad2eed2f-6335-4bc0-87b2-7df7fc4cd82d')
    def test_list_shared_zone(self):
        if not versionutils.is_compatible('2.1', self.api_version,
                                          same_major=False):
            raise self.skipException(
                'Zone share tests require Designate API version 2.1 or newer. '
                'Skipping test_list_shared_zone test.')

        LOG.info('Create zone 1')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_shared_zone_1", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create zone 2')
        zone_2_name = dns_data_utils.rand_zone_name(
            name="list_shared_zone_2", suffix=self.tld_name)
        zone_2 = self.zones_client.create_zone(name=zone_2_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone_2['id'])

        LOG.info('Share zone 2 with alt')
        shared_zone = self.share_zone_client.create_zone_share(
            zone_2['id'], self.alt_zone_client.project_id)[1]
        self.addCleanup(self.share_zone_client.delete_zone_share,
                        zone_2['id'], shared_zone['id'])

        LOG.info('List zones')
        body = self.zones_client.list_zones()[1]

        # Check that primary can see all of the zones
        zone_ids = [item['id'] for item in body['zones']]
        self.assertIn(zone['id'], zone_ids)
        self.assertIn(zone_2['id'], zone_ids)

        LOG.info('List zones as alt')
        body = self.alt_zone_client.list_zones()[1]

        # Make sure alt can only see the zone that was shared with alt
        zone_ids = [item['id'] for item in body['zones']]
        self.assertNotIn(zone['id'], zone_ids)
        self.assertIn(zone_2['id'], zone_ids)

    @decorators.idempotent_id('123f51cb-19d5-48a9-aacc-476742c02141')
    def test_update_zone(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="update_zone", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        # Generate a random description
        description = data_utils.rand_name()

        LOG.info('Update the zone')
        zone = self.zones_client.update_zone(
            zone['id'], description=description)[1]

        LOG.info('Ensure we respond with UPDATE+PENDING')
        self.assertEqual(const.UPDATE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        LOG.info('Ensure we respond with updated values')
        self.assertEqual(description, zone['description'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'ZonesClient', 'update_zone', expected_allowed, True,
            zone['id'], description=description)

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'ZonesClient', 'update_zone', expected_allowed, False,
            zone['id'], description=description,
            headers=self.all_projects_header)
        self.check_CUD_RBAC_enforcement(
            'ZonesClient', 'update_zone', expected_allowed, False,
            zone['id'], description=description,
            headers={'x-auth-sudo-project-id': self.zones_client.project_id})

    @decorators.idempotent_id('3acddc86-62cc-4bfa-8589-b99e5d239bf2')
    @decorators.skip_because(bug="1960487")
    def test_serial_changes_on_update(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="serial_changes_on_update", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name,
                                       wait_until=const.ACTIVE)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info("Update Zone's email")
        update_email = self.zones_client.update_zone(
            zone['id'], email=dns_data_utils.rand_email())[1]
        self.assertNotEqual(
            zone['serial'], update_email['serial'],
            "Failed, expected: 'Serial' is supposed to be changed "
            "on Email update.")

        LOG.info("Update Zone's TTL")
        update_ttl = self.zones_client.update_zone(
            zone['id'], ttl=dns_data_utils.rand_ttl())[1]
        self.assertNotEqual(
            update_email['serial'], update_ttl['serial'],
            "Failed, expected: 'Serial' is supposed to be changed "
            "on TTL update.")

        LOG.info("Update Zone's email and description")
        update_email_description = self.zones_client.update_zone(
            zone['id'],
            email=dns_data_utils.rand_email(),
            description=data_utils.rand_name())[1]
        self.assertNotEqual(
            update_ttl['serial'], update_email_description['serial'],
            "Failed, expect the Serial to change "
            "when the Email and Description are updated")

        LOG.info("Update Zone's description")
        update_description = self.zones_client.update_zone(
            zone['id'], description=data_utils.rand_name())[1]
        self.assertEqual(
            update_email_description['serial'], update_description['serial'],
            "Failed, expect the Serial to not change "
            "when the Description is updated")

    @decorators.idempotent_id('d4ce813e-64a5-11eb-9f43-74e5f9e2a801')
    def test_get_primary_zone_nameservers(self):
        # Create a zone and get the associated "pool_id"
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="get_primary_nameservers", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])
        zone_pool_id = zone['pool_id']

        # Get zone's Name Servers using dedicated API request
        zone_nameservers = self.zones_client.show_zone_nameservers(
            zone['id'])[1]
        zone_nameservers = zone_nameservers['nameservers']
        LOG.info('Zone Name Servers are: {}'.format(zone_nameservers))
        self.assertIsNot(
            0, len(zone_nameservers),
            "Failed - received list of nameservers shouldn't be empty")

        # Use "pool_id" to get the Name Servers used
        pool = self.pool_client.show_pool(zone_pool_id)[1]
        pool_nameservers = pool['ns_records']
        LOG.info('Pool nameservers: {}'.format(pool_nameservers))

        # Make sure that pool's and zone's Name Servers are same
        self.assertCountEqual(
            pool_nameservers, zone_nameservers,
            'Failed - Pool and Zone nameservers should be the same')

        # Test RBAC
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member',
                                     'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'ZonesClient', 'show_zone_nameservers', expected_allowed,
            True, zone['id'])

        # Test with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ZonesClient', 'show_zone_nameservers', expected_allowed,
            False, zone['id'], headers=self.all_projects_header)
        self.check_list_show_RBAC_enforcement(
            'ZonesClient', 'show_zone_nameservers', expected_allowed,
            False, zone['id'],
            headers={'x-auth-sudo-project-id': self.zones_client.project_id})

    @decorators.idempotent_id('9970b632-f2db-11ec-a757-201e8823901f')
    def test_create_zone_ttl_zero(self):
        LOG.info('Create a PRIMARY zone')
        zone_name = dns_data_utils.rand_zone_name(
            name="test_create_zone_ttl_zero", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name, ttl=0)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Ensure we respond with CREATE+PENDING')
        self.assertEqual(const.CREATE, zone['action'])
        self.assertEqual(const.PENDING, zone['status'])

        LOG.info('Fetch the zone, ensure TTL is Zero')
        body = self.zones_client.show_zone(zone['id'])[1]
        self.assertEqual(
            0, body['ttl'],
            "Failed, actual Zone's TTL:{} "
            "is not Zero".format(body['ttl']))


class ZonesAdminTest(BaseZonesTest):
    credentials = ["primary", "admin", "alt"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesAdminTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesAdminTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ZonesClient()
        cls.alt_client = cls.os_alt.dns_v2.ZonesClient()

    @decorators.idempotent_id('f6fe8cce-8b04-11eb-a861-74e5f9e2a801')
    def test_show_zone_impersonate_another_project(self):
        LOG.info('Create zone "A" using primary client')
        zone_name = dns_data_utils.rand_zone_name(
            name="show_zone_impersonate", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('As Alt tenant show zone created by Primary tenant. '
                 'Expected: 404 NotFound')
        self.assertRaises(
            lib_exc.NotFound, self.alt_client.show_zone, uuid=zone['id'])

        LOG.info('As Admin tenant show zone created by Primary tenant. '
                 'Expected: 404 NotFound')
        self.assertRaises(
            lib_exc.NotFound, self.admin_client.show_zone, uuid=zone['id'])

        LOG.info('As Alt tenant show zone created by Primary tenant using '
                 '"x-auth-sudo-project-id" HTTP header. '
                 'Expected: 403 Forbidden')
        self.assertRaises(
            lib_exc.Forbidden, self.alt_client.show_zone, uuid=zone['id'],
            headers={'x-auth-sudo-project-id': zone['project_id']})

        LOG.info('As Admin user impersonate another project '
                 '(using "x-auth-sudo-project-id" HTTP header) to show '
                 'a Primary tenant zone.')
        body = self.admin_client.show_zone(
            uuid=zone['id'], headers={
                'x-auth-sudo-project-id': zone['project_id']})[1]

        LOG.info('Ensure the fetched response matches the impersonated'
                 ' project, it means the ID of a zone "A"')
        self.assertExpected(zone, body, self.excluded_keys)

    @decorators.idempotent_id('e1cf7104-8b06-11eb-a861-74e5f9e2a801')
    def test_list_all_projects_zones(self):

        LOG.info('Create zone "A" using Primary client')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_zone_all_projects_A", suffix=self.tld_name)
        primary_zone = self.zones_client.create_zone(name=zone_name,
                                               wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, primary_zone['id'])

        LOG.info('Create zone "B" using Alt client')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_zone_all_projects_B", suffix=self.tld_name)
        alt_zone = self.alt_client.create_zone(name=zone_name,
                                               wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete, self.alt_client, alt_zone['id'])

        LOG.info('Create zone "C" using Admin client')
        zone_name = dns_data_utils.rand_zone_name(
            name="list_zone_all_projects_C", suffix=self.tld_name)
        admin_zone = self.admin_client.create_zone(
            name=zone_name, project_id="FakeProjectID",
            wait_until=const.ACTIVE)[1]
        self.addCleanup(
            self.wait_zone_delete, self.admin_client, admin_zone['id'],
            headers=self.all_projects_header)

        LOG.info('As admin user list all projects zones')
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        body = self.admin_client.list_zones(
            headers=self.all_projects_header,
            params={'limit': 1000})[1]['zones']
        listed_zone_ids = [item['id'] for item in body]

        LOG.info('Ensure the fetched response includes all zone '
                 'IDs created within the test')

        for id in [primary_zone['id'], alt_zone['id'], admin_zone['id']]:
            self.assertIn(
                id, listed_zone_ids,
                'Failed, id:{} was not found in listed zones:{} '.format(
                    id, listed_zone_ids))


class ZoneOwnershipTest(BaseZonesTest):
    credentials = ["primary", "alt", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZoneOwnershipTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZoneOwnershipTest, cls).setup_clients()
        cls.alt_client = cls.os_alt.dns_v2.ZonesClient()

    @decorators.idempotent_id('5d28580a-a012-4b57-b211-e077b1a01340')
    def test_no_create_duplicate_domain(self):
        LOG.info('Create a zone as a default user')
        zone_name = dns_data_utils.rand_zone_name(
            name="no_create_duplicate", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone as an default with existing domain')
        self.assertRaises(lib_exc.Conflict,
            self.zones_client.create_zone, name=zone['name'])

        LOG.info('Create a zone as an alt user with existing domain')
        self.assertRaises(lib_exc.Conflict,
            self.alt_client.create_zone, name=zone['name'])

    @decorators.idempotent_id('a48776fd-b1aa-4a25-9f09-d1d34cfbb175')
    def test_no_create_subdomain_by_alt_user(self):
        LOG.info('Create a zone as a default user')
        zone_name = dns_data_utils.rand_zone_name(
            name="no_create_subdomain_by_alt", suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone as an alt user with existing subdomain')
        self.assertRaises(lib_exc.Forbidden,
            self.alt_client.create_zone, name='sub.' + zone['name'])
        self.assertRaises(lib_exc.Forbidden,
            self.alt_client.create_zone, name='sub.sub.' + zone['name'])

    @decorators.idempotent_id('f1723d48-c082-43cd-94bf-ebeb5b8c9458')
    def test_no_create_superdomain_by_alt_user(self):
        zone_name = dns_data_utils.rand_zone_name(
            name="no_create_superdomain_by_alt", suffix=self.tld_name)

        LOG.info('Create a zone as a default user')
        zone = self.zones_client.create_zone(name='a.b.' + zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone as an alt user with existing superdomain')
        self.assertRaises(lib_exc.Forbidden,
            self.alt_client.create_zone, name=zone_name)


class ZonesNegativeTest(BaseZonesTest):
    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesNegativeTest, cls).setup_credentials()

    @decorators.idempotent_id('551853c0-8593-11eb-8c8a-74e5f9e2a801')
    def test_no_valid_zone_name(self):
        no_valid_names = ['a' * 1000, '___', '!^%&^#%^!@#', 'ggg', '.a', '']
        for name in no_valid_names:
            LOG.info('Trying to create a zone named: {} '.format(name))
            self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_object', 400,
                self.zones_client.create_zone, name=name)

    @decorators.idempotent_id('551853c0-8593-11eb-8c8a-74e5f9e2a801')
    def test_no_valid_email(self):
        no_valid_emails = [
            'zahlabut#gmail.com', '123456', '___', '', '*&^*^%$']
        for email in no_valid_emails:
            LOG.info(
                'Trying to create a zone using: {} as email'
                ' value: '.format(email))
            self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_object', 400,
                self.zones_client.create_zone, email=email)

    @decorators.idempotent_id('551853c0-8593-11eb-8c8a-74e5f9e2a801')
    def test_no_valid_ttl(self):
        no_valid_ttl = ['zahlabut', -60000,
                        2147483647 + 10]  # Max valid TTL is 2147483647

        for ttl in no_valid_ttl:
            LOG.info(
                'Trying to create a zone using: {} as TTL'
                ' value: '.format(ttl))
            self.assertRaisesDns(
                lib_exc.BadRequest, 'invalid_object', 400,
                self.zones_client.create_zone, ttl=ttl)

    @decorators.idempotent_id('c4d4b92a-86e3-11ee-965f-201e8823901f')
    def test_not_existing_project_id(self):
        LOG.info('Trying to create a zone using not existing project_id')
        self.assertRaises(
            lib_exc.Forbidden, self.zones_client.create_zone,
            project_id=data_utils.rand_uuid())

    @decorators.idempotent_id('a3b0a928-a682-11eb-9899-74e5f9e2a801')
    def test_huge_size_description(self):
        LOG.info('Trying to create a zone using huge size description')
        self.assertRaisesDns(
            lib_exc.BadRequest, 'invalid_object', 400,
            self.zones_client.create_zone,
            description=dns_data_utils.rand_zone_name() * 100)

    @decorators.idempotent_id('49268b24-92de-11eb-9d02-74e5f9e2a801')
    def test_show_not_existing_zone(self):
        LOG.info('Fetch non existing zone')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.zones_client.show_zone(data_utils.rand_uuid()))

    @decorators.idempotent_id('736e3b50-92e0-11eb-9d02-74e5f9e2a801')
    def test_use_invalid_id_to_show_zone(self):
        LOG.info('Fetch the zone using invalid zone ID')
        with self.assertRaisesDns(lib_exc.BadRequest, 'invalid_uuid', 400):
            self.zones_client.show_zone(uuid='zahlabut')

    @decorators.idempotent_id('79921370-92e1-11eb-9d02-74e5f9e2a801')
    def test_delete_non_existing_zone(self):
        LOG.info('Delete non existing zone')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.zones_client.delete_zone(data_utils.rand_uuid()))

    @decorators.idempotent_id('e391e30a-92e0-11eb-9d02-74e5f9e2a801')
    def test_update_non_existing_zone(self):
        LOG.info('Update non existing zone')
        self.assertRaises(lib_exc.NotFound,
            lambda: self.zones_client.update_zone(
                data_utils.rand_uuid(),
                description=data_utils.rand_name()))

    @decorators.idempotent_id('925192f2-0ed8-4591-8fe7-a9fa028f90a0')
    def test_list_zones_dot_json_fails(self):
        uri = self.zones_client.get_uri('zones.json')

        self.assertRaises(lib_exc.NotFound,
            lambda: self.zones_client.get(uri))



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseBlacklistsTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'links']


class BlacklistsAdminTest(BaseBlacklistsTest):

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(BlacklistsAdminTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BlacklistsAdminTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.BlacklistsClient()
        cls.primary_client = cls.os_primary.dns_v2.BlacklistsClient()

    @decorators.idempotent_id('3a7f7564-6bdd-446e-addc-a3475b4c3f71')
    def test_create_blacklist(self):
        LOG.info('Create a blacklist')
        blacklist = {
            'pattern': dns_data_utils.rand_zone_name(),
            'description': data_utils.rand_name(),
        }
        _, body = self.admin_client.create_blacklist(**blacklist)
        self.addCleanup(self.admin_client.delete_blacklist, body['id'])

        self.assertExpected(blacklist, body, self.excluded_keys)

        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement('BlacklistsClient', 'create_blacklist',
                                        expected_allowed, False)

    @decorators.idempotent_id('ea608152-da3c-11eb-b8b8-74e5f9e2a801')
    def test_create_blacklist_invalid_pattern(self):
        patterns = ['', '#(*&^%$%$#@$', 'a' * 1000]
        for pattern in patterns:
            LOG.info(
                'Try to create a blacklist using pattern:{}'.format(pattern))
            self.assertRaises(
                lib_exc.BadRequest, self.admin_client.create_blacklist,
                pattern=pattern)

    @decorators.idempotent_id('664bdaa0-da47-11eb-b8b8-74e5f9e2a801')
    def test_create_blacklist_huge_size_description(self):
        LOG.info('Try to create a blacklist using huge size description')
        self.assertRaises(
            lib_exc.BadRequest, self.admin_client.create_blacklist,
            description='a' * 1000)

    @decorators.idempotent_id('fe9de464-d8d1-11eb-bcdc-74e5f9e2a801')
    def test_create_blacklist_as_primary_fails(self):
        LOG.info('As Primary user, try to create a blacklist')
        self.assertRaises(
            lib_exc.Forbidden, self.primary_client.create_blacklist)

    @decorators.idempotent_id('5bc02942-6225-4619-8f49-2105581a8dd6')
    def test_show_blacklist(self):
        LOG.info('Create a blacklist')
        _, blacklist = self.admin_client.create_blacklist()
        self.addCleanup(self.admin_client.delete_blacklist, blacklist['id'])

        LOG.info('Fetch the blacklist')
        _, body = self.admin_client.show_blacklist(blacklist['id'])

        LOG.info('Ensure the fetched response matches the created blacklist')
        self.assertExpected(blacklist, body, self.excluded_keys)
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'BlacklistsClient', 'show_blacklist', expected_allowed, False,
            blacklist['id'])

    @decorators.idempotent_id('dcea40d9-8d36-43cb-8440-4a842faaef0d')
    def test_delete_blacklist(self):
        LOG.info('Create a blacklist')
        _, blacklist = self.admin_client.create_blacklist()
        self.addCleanup(self.admin_client.delete_blacklist, blacklist['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info('Delete the blacklist')
        _, body = self.admin_client.delete_blacklist(blacklist['id'])

        # A blacklist delete returns an empty body
        self.assertEqual(body.strip(), b"")

        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'BlacklistsClient', 'delete_blacklist', expected_allowed, False,
            blacklist['id'])

    @decorators.idempotent_id('3a2a1e6c-8176-428c-b5dd-d85217c0209d')
    def test_list_blacklists(self):
        LOG.info('Create a blacklist')
        _, blacklist = self.admin_client.create_blacklist()
        self.addCleanup(self.admin_client.delete_blacklist, blacklist['id'])

        LOG.info('List blacklists')
        _, body = self.admin_client.list_blacklists()

        # TODO(pglass): Assert that the created blacklist is in the response
        self.assertGreater(len(body['blacklists']), 0)

        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'BlacklistsClient', 'list_blacklists',
            expected_allowed, [blacklist['id']])

    @decorators.idempotent_id('0063d6ad-9557-49c7-b521-e64a14d4d0d0')
    def test_update_blacklist(self):
        LOG.info('Create a blacklist')
        _, blacklist = self.admin_client.create_blacklist()
        self.addCleanup(self.admin_client.delete_blacklist, blacklist['id'])

        LOG.info('Update the blacklist')
        pattern = dns_data_utils.rand_zone_name()
        description = data_utils.rand_name()
        _, body = self.admin_client.update_blacklist(
            uuid=blacklist['id'],
            pattern=pattern,
            description=description,
        )

        LOG.info('Ensure we response with updated values')
        self.assertEqual(pattern, body['pattern'])
        self.assertEqual(description, body['description'])

        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'BlacklistsClient', 'update_blacklist', expected_allowed, False,
            uuid=blacklist['id'], pattern=pattern, description=description)


class TestBlacklistNotFoundAdmin(BaseBlacklistsTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestBlacklistNotFoundAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestBlacklistNotFoundAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.BlacklistsClient()

    @decorators.idempotent_id('9d65b638-fe98-47a8-853f-fa9244d144cc')
    def test_show_blacklist_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.show_blacklist,
                              data_utils.rand_uuid())
        self.assertBlacklist404(e.resp, e.resp_body)

    @decorators.idempotent_id('a9e12415-5040-4fba-905c-95d201fcfd3b')
    def test_update_blacklist_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.update_blacklist,
                              data_utils.rand_uuid())
        self.assertBlacklist404(e.resp, e.resp_body)

    @decorators.idempotent_id('b1132586-bf06-47a6-9f6f-3bab6a2c1932')
    def test_delete_blacklist_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.delete_blacklist,
                              data_utils.rand_uuid())
        self.assertBlacklist404(e.resp, e.resp_body)

    def assertBlacklist404(self, resp, resp_body):
        self.assertEqual(404, resp.status)
        self.assertEqual(404, resp_body['code'])
        self.assertEqual("blacklist_not_found", resp_body['type'])
        self.assertEqual("Could not find Blacklist", resp_body['message'])


class TestBlacklistInvalidIdAdmin(BaseBlacklistsTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestBlacklistInvalidIdAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestBlacklistInvalidIdAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.BlacklistsClient()

    @decorators.idempotent_id('c7bae53f-2edc-45d8-b254-8a81482728c1')
    def test_show_blacklist_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.show_blacklist,
                              'foo')
        self.assertBlacklistInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('c57b97da-ca87-44b5-9f40-a099937433bf')
    def test_update_blacklist_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.update_blacklist,
                              'foo')
        self.assertBlacklistInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('5d62a026-13e4-48b9-9773-1780660c5920')
    def test_delete_blacklist_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.delete_blacklist,
                              'foo')
        self.assertBlacklistInvalidId(e.resp, e.resp_body)

    def assertBlacklistInvalidId(self, resp, resp_body):
        self.assertEqual(400, resp.status)
        self.assertEqual(400, resp_body['code'])
        self.assertEqual("invalid_uuid", resp_body['type'])
        self.assertEqual("Invalid UUID blacklist_id: foo",
                         resp_body['message'])



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.services.dns.v2.json import base


class ZoneExportsClient(base.DnsClientV2Base):

    @base.handle_errors
    def create_zone_export(self, uuid, params=None,
                           wait_until=False, headers=None):
        """Create a zone export.

        :param uuid: Unique identifier of the zone in UUID format.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param wait_until: Block until the exported zone reaches the
                           desired status
        :param headers (dict): The headers to use for the request.
        :return: Serialized imported zone as a dictionary.
        """

        export_uri = 'zones/{0}/tasks/export'.format(uuid)
        resp, body = self._create_request(
            export_uri, params=params, headers=headers)

        # Create Zone Export should Return a HTTP 202
        self.expected_success(202, resp.status)

        if wait_until:
            waiters.wait_for_zone_export_status(
                self, body['id'], wait_until, headers=headers)

        return resp, body

    @base.handle_errors
    def show_zone_export(self, uuid, params=None, headers=None):
        """Get the zone export task

        :param uuid: Unique identifier of the zone export task in UUID format.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized exported zone as a dictionary.
        """
        return self._show_request(
             'zones/tasks/exports', uuid, params=params, headers=headers)

    @base.handle_errors
    def show_exported_zonefile(self, uuid, params=None, headers=None):

        """Get the exported zone file

        :param uuid: Unique identifier of the zone export task in UUID format.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers: 3 options to send headers:
                       1) If headers dict provided is missing "Accept" key -
                          "{Accept:text/dns}" will be added.
                       2) If header is None -
                          "{Accept:text/dns}" will be sent.
                       3) If function is called with no headers,
                           means empty dict {} -
                          no headers will be sent (empty dictionary)

        :return: Serialized exported zone as a dictionary.
        """

        if headers:
            if 'accept' not in [key.lower() for key in headers.keys()]:
                headers['Accept'] = 'text/dns'
        elif headers is None:
            headers = {'Accept': 'text/dns'}
        else:
            headers = {}

        return self._show_request(
            'zones/tasks/exports/{0}/export'.format(uuid),
            uuid='', headers=headers, params=params)

    @base.handle_errors
    def list_zone_exports(self, params=None, headers=None):
        """List zone export tasks

        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: Serialized exported zone as a list.
        """
        return self._list_request(
            'zones/tasks/exports', params=params, headers=headers)

    @base.handle_errors
    def delete_zone_export(self, uuid, params=None, headers=None):
        """Deletes the zone export task with the specified UUID.

        :param uuid: The unique identifier of the exported zone.
        :param params: A Python dict that represents the query parameters to
                       include in the request URI.
        :param headers (dict): The headers to use for the request.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request(
            'zones/tasks/exports', uuid, params=params, headers=headers)

        # Delete Zone export should Return a HTTP 204
        self.expected_success(204, resp.status)

        return resp, body



# Copyright 2016 Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin import data_utils as dns_data_utils
from designate_tempest_plugin.services.dns.v2.json import base


class BlacklistsClient(base.DnsClientV2Base):

    @base.handle_errors
    def create_blacklist(self, pattern=None, description=None, params=None):
        """Create a blacklist

        :param pattern: The blacklist pattern.
            Default: Random Value
        :param description: A description of the blacklist.
            Default: Random Value
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the created blacklist.
        """
        blacklist = {
            'pattern': pattern or dns_data_utils.rand_zone_name(),
            'description': description or data_utils.rand_name(),
        }

        if pattern == '':
            blacklist['pattern'] = ''

        resp, body = self._create_request('blacklists', blacklist,
                                          params=params)

        self.expected_success(201, resp.status)

        return resp, body

    @base.handle_errors
    def show_blacklist(self, uuid, params=None):
        """Gets a specified blacklist.

        :param uuid: Unique identifier of the blacklist in UUID format.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: Serialized blacklist as a dictionary.
        """
        return self._show_request('blacklists', uuid, params=params)

    @base.handle_errors
    def list_blacklists(self, params=None):
        """Gets a list of blacklists.

        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: Serialized blacklists as a list.
        """
        return self._list_request('blacklists', params=params)

    @base.handle_errors
    def delete_blacklist(self, uuid, params=None):
        """Deletes a blacklist having the specified UUID.

        :param uuid: The unique identifier of the blacklist.
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the response body.
        """
        resp, body = self._delete_request('blacklists', uuid, params=params)

        self.expected_success(204, resp.status)

        return resp, body

    @base.handle_errors
    def update_blacklist(self, uuid, pattern=None, description=None,
                         params=None):
        """Update a blacklist with the specified parameters.

        :param uuid: The unique identifier of the blacklist.
        :param pattern: The blacklist pattern.
            Default: Random Value
        :param description: A description of the blacklist.
            Default: Random Value
        :param params: A Python dict that represents the query paramaters to
                       include in the request URI.
        :return: A tuple with the server response and the updated blacklist.
        """
        blacklist = {
            'pattern': pattern or dns_data_utils.rand_zone_name(),
            'description': description or data_utils.rand_name(),
        }

        resp, body = self._update_request('blacklists', uuid, blacklist,
                                          params=params)

        self.expected_success(200, resp.status)

        return resp, body



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin.tests import base

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseTldTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'links']


class TldAdminTest(BaseTldTest):
    credentials = ["admin", "primary", "alt",
                   "project_reader", "project_member"]

    # Use a TLD suffix unique to this test class.
    local_tld_suffix = '.'.join(["tldadmintest", CONF.dns.tld_suffix])

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TldAdminTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TldAdminTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TldClient()
        cls.primary_client = cls.os_primary.dns_v2.TldClient()

    @classmethod
    def _generate_tld_name(cls, test_name):
        return '.'.join([data_utils.rand_name(name=test_name),
                         cls.local_tld_suffix])

    @decorators.idempotent_id('52a4bb4b-4eff-4591-9dd3-ad98316806c3')
    def test_create_tld(self):
        tld_name = self._generate_tld_name("test_create_tld")
        tld_data = {"name": tld_name, "description": "sample tld"}

        LOG.info('Create a tld')
        tld = self.admin_client.create_tld(tld_data['name'],
                                           tld_data['description'])[1]
        self.addCleanup(self.admin_client.delete_tld, tld['id'])

        self.assertEqual(tld_name, tld['name'])

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement('TldClient', 'create_tld',
                                        expected_allowed, False)

    @decorators.idempotent_id('961bd2e8-d4d0-11eb-b8ee-74e5f9e2a801')
    def test_create_duplicated_tlds(self):
        tld_name = self._generate_tld_name("test_create_duplicated_tlds")
        tld_data = {"name": tld_name,
                    "description": "test_create_duplicated_tlds"}

        LOG.info('Create a first "org" TLD')
        tld = self.admin_client.create_tld(
            tld_data['name'], tld_data['description'])[1]
        self.addCleanup(self.admin_client.delete_tld, tld['id'])
        self.assertEqual(tld_name, tld['name'])

        LOG.info('Try to create a second "org" TLD')
        self.assertRaises(
            lib_exc.Conflict, self.admin_client.create_tld,
            tld_name=tld_data['name'],
            description=tld_data['description'])

    @decorators.idempotent_id('0c0ab92e-d4db-11eb-b8ee-74e5f9e2a801')
    def test_create_multiply_tlds(self):
        for _dummy in range(0, 2):
            tld_name = self._generate_tld_name("test_create_multiply_tlds")
            tld_data = {"name": tld_name,
                        "description": "test_create_multiply_tlds"}
            LOG.info('Create a "{}" TLD'.format(tld_name))
            tld = self.admin_client.create_tld(
                tld_data['name'], tld_data['description'])[1]
            self.addCleanup(self.admin_client.delete_tld, tld['id'])
            self.assertEqual(tld_name, tld['name'])

    @decorators.idempotent_id('52a4bb4b-4eff-4591-9dd3-ad98316806c3')
    def test_create_invalid_tld(self):
        invalid_tlds = ['   ', 'fff.', '^&$', 't' * 1000, '@p%']
        for tld_name in invalid_tlds:
            tld_data = {
                "name": tld_name, "description": "test_create_invalid_tld"}
            LOG.info('Create a "{}" TLD'.format(tld_name))
            self.assertRaises(
                lib_exc.BadRequest, self.admin_client.create_tld,
                tld_name=tld_data['name'], description=tld_data['description'])
        LOG.info('Create a "{}" TLD with huge size description.')
        self.assertRaises(
            lib_exc.BadRequest, self.admin_client.create_tld,
            tld_name='org', description='test_create_invalid_tld' * 1000)

    @decorators.idempotent_id('757019c0-d4e2-11eb-b8ee-74e5f9e2a801')
    def test_create_tld_as_primary_user(self):
        tld_data = {
            "name": "org", "description": "test_create_tld_as_primary_user"}
        LOG.info('Try to create a TLD as primary user.')
        self.assertRaises(
            lib_exc.Forbidden, self.primary_client.create_tld,
            tld_name=tld_data['name'], description=tld_data['description'])

    @decorators.idempotent_id('271af08c-2603-4f61-8eb1-05887b74e25a')
    def test_show_tld(self):
        tld_name = self._generate_tld_name("test_show_tld")
        tld_data = {
                     "name": tld_name,
                     "description": "sample tld"}

        LOG.info('Create a tld')
        tld = self.admin_client.create_tld(tld_data['name'],
                                           tld_data['description'])[1]
        self.addCleanup(self.admin_client.delete_tld, tld['id'])

        LOG.info('Fetch the tld')
        body = self.admin_client.show_tld(tld['id'])[1]

        LOG.info('Ensure the fetched response matches the created tld')
        self.assertExpected(tld, body, self.excluded_keys)

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'TldClient', 'show_tld', expected_allowed, False, tld['id'])

    @decorators.idempotent_id('26708cb8-7126-48a7-9424-1c225e56e609')
    def test_delete_tld(self):
        LOG.info('Create a tld')
        tld_name = self._generate_tld_name("test_delete_tld")
        tld = self.admin_client.create_tld(tld_name)[1]
        self.addCleanup(self.admin_client.delete_tld, tld['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info('Delete the tld')
        self.admin_client.delete_tld(tld['id'])

        self.assertRaises(lib_exc.NotFound, self.admin_client.show_tld,
                          tld['id'])

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement('TldClient', 'delete_tld',
                                        expected_allowed, False, tld['id'])

    @decorators.idempotent_id('95b13759-c85c-4791-829b-9591ca15779d')
    def test_list_tlds(self):
        LOG.info('List tlds')
        tld_name = self._generate_tld_name("test_list_tlds")
        tld = self.admin_client.create_tld(tld_name)[1]
        self.addCleanup(self.admin_client.delete_tld, tld['id'],
                        ignore_errors=lib_exc.NotFound)

        body = self.admin_client.list_tlds()[1]

        self.assertGreater(len(body['tlds']), 0)

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'TldClient', 'list_tlds', expected_allowed, [tld['id']],
            params={'limit': 1000})

    @decorators.idempotent_id('1a233812-48d9-4d15-af5e-9961744286ff')
    def test_update_tld(self):
        tld_name = self._generate_tld_name("test_update_tld")
        tld = self.admin_client.create_tld(tld_name)[1]
        self.addCleanup(self.admin_client.delete_tld, tld['id'])

        tld_name_2 = self._generate_tld_name("test_update_tld")

        tld_data = {
                     "name": tld_name_2,
                     "description": "Updated description"
        }

        LOG.info('Update the tld')
        patch_tld = self.admin_client.update_tld(tld['id'],
                       tld_data['name'], tld_data['description'])[1]

        self.assertEqual(tld_name_2, patch_tld["name"])
        self.assertEqual(tld_data["description"], patch_tld["description"])

        # Test RBAC
        expected_allowed = ['os_admin']

        self.check_CUD_RBAC_enforcement(
            'TldClient', 'update_tld', expected_allowed, False, tld['id'],
            tld_data['name'], tld_data['description'])

    @decorators.idempotent_id('8116dcf5-a329-47d1-90be-5ff32f299c53')
    def test_list_tlds_dot_json_fails(self):
        uri = self.admin_client.get_uri('tlds.json')

        self.assertRaises(lib_exc.NotFound,
            lambda: self.admin_client.get(uri))


class TestTldNotFoundAdmin(BaseTldTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestTldNotFoundAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestTldNotFoundAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TldClient()

    @decorators.idempotent_id('b237d5ee-0d76-4294-a3b6-c2f8bf4b0e30')
    def test_show_tld_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.show_tld,
                              data_utils.rand_uuid())
        self.assertTld404(e.resp, e.resp_body)

    @decorators.idempotent_id('3d128772-7f52-4473-b569-51ae8294667b')
    def test_update_tld_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.update_tld,
                              data_utils.rand_uuid())
        self.assertTld404(e.resp, e.resp_body)

    @decorators.idempotent_id('18e465e7-5c7d-4775-acef-bd12a8db1095')
    def test_delete_tld_404(self):
        e = self.assertRaises(lib_exc.NotFound,
                              self.admin_client.delete_tld,
                              data_utils.rand_uuid())
        self.assertTld404(e.resp, e.resp_body)

    def assertTld404(self, resp, resp_body):
        self.assertEqual(404, resp.status)
        self.assertEqual(404, resp_body['code'])
        self.assertEqual("tld_not_found", resp_body['type'])
        self.assertEqual("Could not find Tld", resp_body['message'])


class TestTldInvalidIdAdmin(BaseTldTest):

    credentials = ["admin", "primary"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(TestTldInvalidIdAdmin, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(TestTldInvalidIdAdmin, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.TldClient()

    @decorators.idempotent_id('f9ec0730-57ff-4720-8d06-e11d377c7cfc')
    def test_show_tld_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.show_tld,
                              'foo')
        self.assertTldInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('13dc6518-b479-4502-90f5-f5a5ecc8b1fb')
    def test_update_tld_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.update_tld,
                              'foo')
        self.assertTldInvalidId(e.resp, e.resp_body)

    @decorators.idempotent_id('6a6fc9db-9a73-4ffc-831a-172e1cbc7394')
    def test_delete_tld_invalid_uuid(self):
        e = self.assertRaises(lib_exc.BadRequest,
                              self.admin_client.delete_tld,
                              'foo')
        self.assertTldInvalidId(e.resp, e.resp_body)

    def assertTldInvalidId(self, resp, resp_body):
        self.assertEqual(400, resp.status)
        self.assertEqual(400, resp_body['code'])
        self.assertEqual("invalid_uuid", resp_body['type'])
        self.assertEqual("Invalid UUID tld_id: foo",
                         resp_body['message'])



# Copyright 2021 Red Hat, Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from oslo_log import log as logging
from tempest import config
from tempest.lib import exceptions
from tempest import test

CONF = config.CONF
LOG = logging.getLogger(__name__)


class RBACTestsMixin(test.BaseTestCase):

    def _get_client_method(self, cred_obj, client_str, method_str):
        """Get requested method from registered clients in Tempest."""
        dns_clients = getattr(cred_obj, 'dns_v2')
        client = getattr(dns_clients, client_str)
        client_obj = client()
        method = getattr(client_obj, method_str)
        return method

    def _get_client_project_id(self, cred_obj, client_str):
        """Get project ID for the credential."""
        dns_clients = getattr(cred_obj, 'dns_v2')
        client = getattr(dns_clients, client_str)
        client_obj = client()
        return client_obj.project_id

    def _check_allowed(self, client_str, method_str, allowed_list,
                       with_project, *args, **kwargs):
        """Test an API call allowed RBAC enforcement.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param allowed_list: The list of credentials expected to be
                             allowed.  Example: ['primary'].
        :param with_project: When true, pass the project ID to the call.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """
        for cred in allowed_list:
            try:
                cred_obj = getattr(self, cred)
            except AttributeError:
                self.fail('Credential {} "expected_allowed" for RBAC '
                          'testing was not created by tempest '
                          'credentials setup. This is likely a bug in the '
                          'test.'.format(cred))
            method = self._get_client_method(cred_obj, client_str, method_str)
            project_id = self._get_client_project_id(cred_obj, client_str)
            try:
                if with_project:
                    method(project_id, *args, **kwargs)
                else:
                    method(*args, **kwargs)
            except exceptions.Forbidden as e:
                self.fail('Method {}.{} failed to allow access via RBAC using '
                          'credential {}. Error: {}'.format(
                              client_str, method_str, cred, str(e)))
            except exceptions.NotFound as e:
                self.fail('Method {}.{} failed to allow access via RBAC using '
                          'credential {}. Error: {}'.format(
                              client_str, method_str, cred, str(e)))

    def _check_disallowed(self, client_str, method_str, allowed_list,
                          expect_404, with_project, *args, **kwargs):
        """Test an API call disallowed RBAC enforcement.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param allowed_list: The list of credentials expected to be
                             allowed.  Example: ['primary'].
        :param expect_404: When True, 404 responses are considered ok.
        :param with_project: When true, pass the project ID to the call.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """
        expected_disallowed = (set(self.allocated_credentials) -
                               set(allowed_list))
        for cred in expected_disallowed:
            cred_obj = getattr(self, cred)
            method = self._get_client_method(cred_obj, client_str, method_str)
            project_id = self._get_client_project_id(cred_obj, client_str)

            # Unfortunately tempest uses testtools assertRaises[1] which means
            # we cannot use the unittest assertRaises context[2] with msg= to
            # give a useful error.
            # Also, testtools doesn't work with subTest[3], so we can't use
            # that to expose the failing credential.
            # This all means the exception raised testtools assertRaises
            # is less than useful.
            # TODO(johnsom) Remove this try block once testtools is useful.
            # [1] https://testtools.readthedocs.io/en/latest/
            #     api.html#testtools.TestCase.assertRaises
            # [2] https://docs.python.org/3/library/
            #     unittest.html#unittest.TestCase.assertRaises
            # [3] https://github.com/testing-cabal/testtools/issues/235
            try:
                if with_project:
                    method(project_id, *args, **kwargs)
                else:
                    method(*args, **kwargs)
            except exceptions.Forbidden:
                continue
            except exceptions.NotFound:
                # Some APIs hide that the resource exists by returning 404
                # on permission denied.
                if expect_404:
                    continue
                raise
            self.fail('Method {}.{} failed to deny access via RBAC using '
                      'credential {}.'.format(client_str, method_str, cred))

    def check_list_show_RBAC_enforcement(self, client_str, method_str,
                                         expected_allowed, expect_404,
                                         *args, **kwargs):
        """Test list or show API call RBAC enforcement.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param expected_allowed: The list of credentials expected to be
                                 allowed.  Example: ['primary'].
        :param expect_404: When True, 404 responses are considered ok.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """

        allowed_list = copy.deepcopy(expected_allowed)

        # #### Test that disallowed credentials cannot access the API.
        self._check_disallowed(client_str, method_str, allowed_list,
                               expect_404, False, *args, **kwargs)

        # #### Test that allowed credentials can access the API.
        self._check_allowed(client_str, method_str, allowed_list, False,
                            *args, **kwargs)

    def check_list_show_with_ID_RBAC_enforcement(self, client_str, method_str,
                                                 expected_allowed, expect_404,
                                                 *args, **kwargs):
        """Test list or show API call passing the project ID RBAC enforcement.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param expected_allowed: The list of credentials expected to be
                                 allowed.  Example: ['primary'].
        :param expect_404: When True, 404 responses are considered ok.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """

        allowed_list = copy.deepcopy(expected_allowed)

        # #### Test that disallowed credentials cannot access the API.
        self._check_disallowed(client_str, method_str, allowed_list,
                               expect_404, True, *args, **kwargs)

        # #### Test that allowed credentials can access the API.
        self._check_allowed(client_str, method_str, allowed_list, True,
                            *args, **kwargs)

    def check_CUD_RBAC_enforcement(self, client_str, method_str,
                                   expected_allowed, expect_404,
                                   *args, **kwargs):
        """Test an API create/update/delete call RBAC enforcement.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param expected_allowed: The list of credentials expected to be
                                 allowed.  Example: ['primary'].
        :param expect_404: When True, 404 responses are considered ok.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """

        allowed_list = copy.deepcopy(expected_allowed)

        # #### Test that disallowed credentials cannot access the API.
        self._check_disallowed(client_str, method_str, allowed_list,
                               expect_404, False, *args, **kwargs)

    def check_list_RBAC_enforcement_count(
            self, client_str, method_str, expected_allowed, expected_count,
            *args, **kwargs):
        """Test an API list call RBAC enforcement result count.

        List APIs will return the object list for the project associated
        with the token used to access the API. This means most credentials
        will have access, but will get differing results.

        This test will query the list API using a list of credentials and
        will validate that only the expected count of results are returned.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param expected_allowed: The list of credentials expected to be
                                 allowed.  Example: ['primary'].
        :param expected_count: The number of results expected in the list
                               returned from the API.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """

        allowed_list = copy.deepcopy(expected_allowed)

        for cred in allowed_list:
            try:
                cred_obj = getattr(self, cred)
            except AttributeError:
                self.fail('Credential {} "expected_allowed" for RBAC '
                          'testing was not created by tempest '
                          'credentials setup. This is likely a bug in the '
                          'test.'.format(cred))
            method = self._get_client_method(cred_obj, client_str, method_str)
            try:
                # Get the result body
                result = method(*args, **kwargs)[1]
            except exceptions.Forbidden as e:
                self.fail('Method {}.{} failed to allow access via RBAC using '
                          'credential {}. Error: {}'.format(
                              client_str, method_str, cred, str(e)))
            # Remove the root element
            result_objs = next(iter(result.values()))

            self.assertEqual(expected_count, len(result_objs),
                             message='Credential {} saw {} objects when {} '
                             'was expected.'.format(cred, len(result_objs),
                                                    expected_count))

    def check_list_IDs_RBAC_enforcement(
            self, client_str, method_str, expected_allowed, expected_ids,
            *args, **kwargs):
        """Test an API list call RBAC enforcement result contains IDs.

        List APIs will return the object list for the project associated
        with the token used to access the API. This means most credentials
        will have access, but will get differing results.

        This test will query the list API using a list of credentials and
        will validate that the expected object Ids in included in the results.

        :param client_str: The service client to use for the test, without the
                           credential.  Example: 'ZonesClient'
        :param method_str: The method on the client to call for the test.
                           Example: 'list_zones'
        :param expected_allowed: The list of credentials expected to be
                                 allowed.  Example: ['primary'].
        :param expected_ids: The list of object IDs to validate are included
                             in the returned list from the API.
        :param args: Any positional parameters needed by the method.
        :param kwargs: Any named parameters needed by the method.
        :raises AssertionError: Raised if the RBAC tests fail.
        :raises Forbidden: Raised if a credential that should have access does
                           not and is denied.
        :raises InvalidScope: Raised if a credential that should have the
                              correct scope for access is denied.
        :returns: None on success
        """

        allowed_list = copy.deepcopy(expected_allowed)

        for cred in allowed_list:
            try:
                cred_obj = getattr(self, cred)
            except AttributeError:
                self.fail('Credential {} "expected_allowed" for RBAC '
                          'testing was not created by tempest '
                          'credentials setup. This is likely a bug in the '
                          'test.'.format(cred))
            method = self._get_client_method(cred_obj, client_str, method_str)
            try:
                # Get the result body
                result = method(*args, **kwargs)[1]
            except exceptions.Forbidden as e:
                self.fail('Method {}.{} failed to allow access via RBAC using '
                          'credential {}. Error: {}'.format(
                              client_str, method_str, cred, str(e)))
            # Remove the root element
            result_objs = next(iter(result.values()))

            result_ids = [result_obj["id"] for result_obj in result_objs]
            self.assertTrue(set(expected_ids).issubset(set(result_ids)))



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.common.utils import data_utils

from designate_tempest_plugin.tests import base
from designate_tempest_plugin.common import waiters
from designate_tempest_plugin.common import constants as const
from designate_tempest_plugin import data_utils as dns_data_utils

CONF = config.CONF
LOG = logging.getLogger(__name__)


class BaseZoneExportsTest(base.BaseDnsV2Test):
    excluded_keys = ['created_at', 'updated_at', 'version', 'links',
                     'status', 'location']

    @classmethod
    def setup_clients(cls):
        super(BaseZoneExportsTest, cls).setup_clients()

        cls.admin_tld_client = cls.os_admin.dns_v2.TldClient()

    @classmethod
    def resource_setup(cls):
        super(BaseZoneExportsTest, cls).resource_setup()

        # Make sure we have an allowed TLD available
        tld_name = dns_data_utils.rand_zone_name(name="BaseZoneExportsTest")
        cls.tld_name = f".{tld_name}"
        cls.class_tld = cls.admin_tld_client.create_tld(tld_name=tld_name[:-1])

    @classmethod
    def resource_cleanup(cls):
        cls.admin_tld_client.delete_tld(cls.class_tld[1]['id'])
        super(BaseZoneExportsTest, cls).resource_cleanup()


class ZonesExportTest(BaseZoneExportsTest):
    credentials = ["primary", "admin", "alt",
                   "project_member", "project_reader"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesExportTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesExportTest, cls).setup_clients()
        cls.admin_client = cls.os_admin.dns_v2.ZoneExportsClient()
        cls.alt_zone_client = cls.os_alt.dns_v2.ZonesClient()
        cls.client = cls.os_primary.dns_v2.ZoneExportsClient()
        cls.alt_client = cls.os_alt.dns_v2.ZoneExportsClient()

    def _create_zone_export(self, test_name):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name=test_name, suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone export')
        zone_export = self.client.create_zone_export(zone['id'])[1]
        self.addCleanup(self.client.delete_zone_export, zone_export['id'])
        waiters.wait_for_zone_export_status(
            self.client, zone_export['id'], const.COMPLETE)
        return zone, zone_export

    @decorators.idempotent_id('2dd8a9a0-98a2-4bf6-bb51-286583b30f40')
    def test_create_zone_export(self):
        zone, zone_export = self._create_zone_export('create_zone_export')

        LOG.info('Ensure we respond with PENDING')
        self.assertEqual(const.PENDING, zone_export['status'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary', 'os_alt']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.append('os_project_member')

        self.check_CUD_RBAC_enforcement(
            'ZoneExportsClient', 'create_zone_export', expected_allowed, True,
            zone['id'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('2d29a2a9-1941-4b7e-9d8a-ad6c2140ea68')
    def test_show_zone_export(self):
        zone_export = self._create_zone_export('show_zone_export')[1]

        LOG.info('Re-Fetch the zone export')
        body = self.client.show_zone_export(zone_export['id'])[1]

        LOG.info('Ensure the fetched response matches the zone export')
        self.assertExpected(zone_export, body, self.excluded_keys)

        # Test RBAC
        expected_allowed = ['os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member',
                                     'os_project_reader'])

        self.check_list_show_RBAC_enforcement(
            'ZoneExportsClient', 'show_zone_export', expected_allowed, True,
            zone_export['id'])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ZoneExportsClient', 'show_zone_export', expected_allowed, True,
            zone_export['id'], headers=self.all_projects_header)

    @decorators.idempotent_id('fb04507c-9600-11eb-b1cd-74e5f9e2a801')
    def test_show_zone_export_impersonate_another_project(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name='show_zone_export_impersonate', suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone export using primary client')
        resp, zone_export = self.client.create_zone_export(zone['id'])
        self.addCleanup(self.client.delete_zone_export, zone_export['id'])

        LOG.info('Impersonate "primary" client, to show created zone exports')
        body = self.admin_client.show_zone_export(uuid=None, headers={
            'x-auth-sudo-project-id': zone['project_id']})[1]['exports']
        listed_export_ids = [item['id'] for item in body]

        LOG.info('Ensure that the fetched response, contains the ID '
                 'for a zone export created by primary client.')
        self.assertIn(
            zone_export['id'], listed_export_ids,
            'Failed, expected ID:{} was not found in listed export zones '
            'for a primary client: {}'.format(
                zone_export['id'], listed_export_ids))

        # Test RBAC with x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_show_RBAC_enforcement(
            'ZoneExportsClient', 'show_zone_export', expected_allowed, True,
            zone_export['id'],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('97234f00-8bcb-43f8-84dd-874f8bc4a80e')
    def test_delete_zone_export(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(
            name='delete_zone_export', suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)

        LOG.info('Create a zone export')
        _, zone_export = self.client.create_zone_export(zone['id'])

        # Test RBAC
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'ZoneExportsClient', 'delete_zone_export', expected_allowed, True,
            zone_export['id'])

        # Test RBAC with x-auth-all-projects and x-auth-sudo-project-id header
        expected_allowed = ['os_admin', 'os_primary']
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed.extend(['os_project_member'])

        self.check_CUD_RBAC_enforcement(
            'ZoneExportsClient', 'delete_zone_export', expected_allowed, False,
            zone_export['id'], headers=self.all_projects_header)

        self.check_CUD_RBAC_enforcement(
            'ZoneExportsClient', 'delete_zone_export', expected_allowed, False,
            zone_export['id'],
            headers={'x-auth-sudo-project-id': self.client.project_id})

        LOG.info('Delete the zone export')
        _, body = self.client.delete_zone_export(zone_export['id'])

        LOG.info('Ensure the zone export has been successfully deleted')
        self.assertRaises(
            lib_exc.NotFound,
            self.client.show_zone_export, zone_export['id'])

    @decorators.idempotent_id('476bfdfe-58c8-46e2-b376-8403c0fff440')
    def test_list_zone_exports(self):
        export = self._create_zone_export('list_zone_exports')[1]

        LOG.info('List zone exports')
        body = self.client.list_zone_exports()[1]

        self.assertGreater(len(body['exports']), 0)

        # Test RBAC - Users that are allowed to call list, but should get
        #             zero zones.
        if CONF.dns_feature_enabled.enforce_new_defaults:
            expected_allowed = ['os_admin']
        else:
            expected_allowed = ['os_alt']

        self.check_list_RBAC_enforcement_count(
            'ZoneExportsClient', 'list_zone_exports', expected_allowed, 0)

        # Test that users who should see the zone, can see it.
        expected_allowed = ['os_primary']

        self.check_list_IDs_RBAC_enforcement(
            'ZoneExportsClient', 'list_zone_exports',
            expected_allowed, [export['id']])

        # Test RBAC with x-auth-sudo-project-id header
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'ZoneExportsClient', 'list_zone_exports',
            expected_allowed, [export['id']],
            headers={'x-auth-sudo-project-id': self.client.project_id})

    @decorators.idempotent_id('f34e7f34-9613-11eb-b1cd-74e5f9e2a801')
    def test_list_zone_exports_all_projects(self):
        LOG.info('Create a primary zone and its export')
        zone_name = dns_data_utils.rand_zone_name(
            name='list_zone_exports_all_projects', suffix=self.tld_name)
        primary_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, primary_zone['id'])
        primary_export = self.client.create_zone_export(primary_zone['id'])[1]
        self.addCleanup(self.client.delete_zone_export, primary_export['id'])

        LOG.info('Create an alt zone and its export')
        alt_zone_name = dns_data_utils.rand_zone_name(
            name='list_zone_exports_all_projects_alt', suffix=self.tld_name)
        alt_zone = self.alt_zone_client.create_zone(name=alt_zone_name)[1]
        self.addCleanup(
            self.wait_zone_delete, self.alt_zone_client, alt_zone['id'])
        alt_export = self.alt_client.create_zone_export(alt_zone['id'])[1]
        self.addCleanup(self.alt_client.delete_zone_export, alt_export['id'])

        LOG.info('As admin user list zone exports for all projects')
        # Note: This is an all-projects list call, so other tests running
        #       in parallel will impact the list result set. Since the default
        #       pagination limit is only 20, we set a param limit of 1000 here.
        listed_exports_ids = [
            item['id'] for item in self.admin_client.list_zone_exports(
                headers=self.all_projects_header,
                params={'limit': 1000})[1]['exports']]

        LOG.info('Make sure that all previously created zone '
                 'export IDs are listed')
        for id in [primary_export['id'], alt_export['id']]:
            self.assertIn(
                id, listed_exports_ids,
                'Failed, expected ID:{} was not found in '
                'listed IDs:{}'.format(id, listed_exports_ids))

        # Test RBAC with x-auth-all-projects
        expected_allowed = ['os_admin']

        self.check_list_IDs_RBAC_enforcement(
            'ZoneExportsClient', 'list_zone_exports', expected_allowed,
            [alt_export['id']], headers=self.all_projects_header)

    @decorators.idempotent_id('e4a11a14-9aaa-11eb-be59-74e5f9e2a801')
    def test_list_zone_exports_filter_results(self):

        LOG.info('Create a primary zone and its export')
        zone_name = dns_data_utils.rand_zone_name(
            name='list_zone_exports_filter', suffix=self.tld_name)
        primary_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, primary_zone['id'])
        primary_export = self.client.create_zone_export(primary_zone['id'])[1]
        self.addCleanup(self.client.delete_zone_export, primary_export['id'])

        LOG.info('Create an alt zone, its export and delete it')
        zone_name = dns_data_utils.rand_zone_name(
            name='list_zone_exports_filter_alt', suffix=self.tld_name)
        alt_zone = self.alt_zone_client.create_zone(name=zone_name)[1]
        self.addCleanup(
            self.wait_zone_delete, self.alt_zone_client, alt_zone['id'])
        alt_export = self.alt_client.create_zone_export(alt_zone['id'])[1]
        self.alt_client.delete_zone_export(alt_export['id'])
        LOG.info('Ensure the zone export has been successfully deleted')
        self.assertRaises(
            lib_exc.NotFound,
            self.alt_client.show_zone_export,
            alt_export['id'])

        LOG.info('Filter out "export zones" in status:ZAHLABUT,'
                 ' expected: empty list')
        self.assertEqual(
            [], self.admin_client.list_zone_exports(
                headers=self.all_projects_header,
                params={'status': 'ZAHLABUT'})[1]['exports'],
            'Failed, filtered result is expected to be empty.')

        LOG.info('Filter out "export zones" with message:ZABABUN,'
                 ' expected: empty list')
        self.assertEqual(
            [], self.admin_client.list_zone_exports(
                headers=self.all_projects_header,
                params={'message': 'ZABABUN'})[1]['exports'],
            'Failed, filtered result is expected to be empty.')

        LOG.info('Filter out "export zones" that have been created for '
                 'a primary zone. Expected: single zone export is listed')
        self.assertEqual(
            1, len(self.admin_client.list_zone_exports(
                headers=self.all_projects_header,
                params={'zone_id': primary_zone['id']})[1]['exports']),
            'Failed, filtered result should contain a single zone '
            '(primary zone export)')

        LOG.info('Filter out "export zones" that have been created for '
                 'an alt zone expected: empty list (it was deleted)')
        self.assertEqual(
            [], self.admin_client.list_zone_exports(
                headers=self.all_projects_header,
                params={'zone_id': alt_zone['id']})[1]['exports'],
            'Failed, filtered result should be empty.')


class ZonesExportTestNegative(BaseZoneExportsTest):
    credentials = ["primary", "alt", "admin"]

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these test.
        cls.set_network_resources()
        super(ZonesExportTestNegative, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(ZonesExportTestNegative, cls).setup_clients()
        cls.client = cls.os_primary.dns_v2.ZoneExportsClient()
        cls.alt_client = cls.os_alt.dns_v2.ZoneExportsClient()

    def _create_zone_export(self, test_name):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(name=test_name,
                                                  suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'])

        LOG.info('Create a zone export')
        zone_export = self.client.create_zone_export(zone['id'])[1]
        self.addCleanup(self.client.delete_zone_export, zone_export['id'])
        waiters.wait_for_zone_export_status(
            self.client, zone_export['id'], const.COMPLETE)
        return zone, zone_export

    @decorators.idempotent_id('76ab8ec4-95fd-11eb-b1cd-74e5f9e2a801')
    def test_create_zone_export_using_invalid_zone_id(self):
        self.assertRaises(
            lib_exc.NotFound, self.client.create_zone_export,
            'e35bc796-9841-11eb-898b-74e5f9e2a801')

    @decorators.idempotent_id('943dad4a-9617-11eb-b1cd-74e5f9e2a801')
    def test_export_not_your_zone(self):
        LOG.info('Create a primary zone.')
        zone_name = dns_data_utils.rand_zone_name(name='export_not_your_zone',
                                                  suffix=self.tld_name)
        primary_zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(
            self.wait_zone_delete, self.zones_client, primary_zone['id'])

        LOG.info('Make sure that "404 NotFound" status code is raised.')
        self.assertRaises(
            lib_exc.NotFound, self.alt_client.create_zone_export,
            primary_zone['id'])

    @decorators.idempotent_id('518dc308-9604-11eb-b1cd-74e5f9e2a801')
    def test_create_zone_export_using_deleted_zone(self):
        LOG.info('Create a zone')
        zone_name = dns_data_utils.rand_zone_name(name='export_deleted_zone',
                                                  suffix=self.tld_name)
        zone = self.zones_client.create_zone(name=zone_name)[1]
        self.addCleanup(self.wait_zone_delete, self.zones_client, zone['id'],
                        ignore_errors=lib_exc.NotFound)
        LOG.info("Delete the zone and wait till it's done.")
        self.zones_client.delete_zone(zone['id'])[1]
        self.wait_zone_delete(self.zones_client, zone['id'])

        LOG.info('Ensure we respond with NotFound exception')
        self.assertRaises(
            lib_exc.NotFound, self.client.create_zone_export, zone['id'])

    @decorators.idempotent_id('9a878646-f66b-4fa4-ae95-f3ac3f8e3d31')
    def test_show_zonefile_using_not_existing_zone_export_id(self):
        LOG.info('Expected: 404 Not Found zone export')
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_exported_zonefile,
                          data_utils.rand_uuid())

    @decorators.idempotent_id('52a1fee0-c338-4ed9-b9f9-41ee7fd73375')
    def test_show_zonefile_not_supported_accept_value(self):
        zone, zone_export = self._create_zone_export(
            'show_zonefile_bad_accept')
        # Tempest-lib _error_checker will raise UnexpectedResponseCode
        e = self.assertRaises(
            lib_exc.NotAcceptable, self.client.show_exported_zonefile,
            zone_export['id'], headers={'Accept': 'image/jpeg'})
        self.assertEqual(406, e.resp.status,
                         "Failed, actual response code is:{0}"
                         "but expected is: 406".format(e.resp.status))
